/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.generator;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;
import java.util.BitSet;

// Zeus imports
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.Generator;
import org.enhydra.zeus.ZeusDefaults;
import org.enhydra.zeus.ZeusException;
import org.enhydra.zeus.binding.AtomicProperty;
import org.enhydra.zeus.binding.BaseBinding;
import org.enhydra.zeus.binding.Container;
import org.enhydra.zeus.binding.Property;
import org.enhydra.zeus.util.CapitalizationUtils;
import org.enhydra.zeus.util.ClassUtils;
import org.enhydra.zeus.util.NamingUtils;
import org.enhydra.zeus.util.XMLName;

/**
 * <p>
 *  <code>{@link Generator}</code> is the portion of Zeus that will convert
 *    a set of constraints, visible through a <code>{@link Binding}</code>,
 *    and output them as generated Java classes (to a 
 *    <code>File</code> [directory]).
 * </p><p>
 *  <code>SimpleGenerator</code> is the simplest form of a 
 *    <code>Generator</code> and takes in either a single <code>Binding</code>
 *    or a <code>List</code> of them, and directly generates Java classes.
 * </p>
 *
 * @author Brett McLaughlin
 * @author Maciej Zawadzki
 * @author Sean Ogle
 */
public class SimpleGenerator implements Generator {

    /** A list of bindings to ignore */
    protected List ignoredBindings;
    
    /** A list of classes to import for all unmarshallers */
    protected List globalUnmarshallerImports;
    
    /** A list of classes to import for all interfaces */
    protected List globalInterfaceImports;
    
    /** A list of classes to import for all implementations */
    protected List globalImplementationImports;
    
    /** This instances's collapsed elements */
    protected List collapsedElements;
    
    /** Base directory for generated files. */
    protected File outputDir;
    
    /** Keeps track of whether the Generator needs to be reinitialized. */
    protected boolean initialized;

    /**
     * <p>
     *  This will construct a new instance of <code>SimpleGenerator</code>.
     * </p>
     */
    public SimpleGenerator() {
        initialized = false;
        ignoredBindings = new LinkedList();
        globalUnmarshallerImports = new LinkedList();
        globalInterfaceImports = new LinkedList();
        globalImplementationImports = new LinkedList();
        collapsedElements = new LinkedList();
        
        // Add some required imports for all unmarshallers
        globalUnmarshallerImports.add("java.io.File");
        globalUnmarshallerImports.add("java.io.FileReader");
        globalUnmarshallerImports.add("java.io.InputStream");
        globalUnmarshallerImports.add("java.io.InputStreamReader");
        globalUnmarshallerImports.add("java.io.IOException");
        globalUnmarshallerImports.add("java.io.Reader");
        globalUnmarshallerImports.add("org.xml.sax.EntityResolver");
        globalUnmarshallerImports.add("org.xml.sax.ErrorHandler");
        
        // Add some required imports for all interfaces
        globalInterfaceImports.add("java.io.File");
        globalInterfaceImports.add("java.io.IOException");
        globalInterfaceImports.add("java.io.OutputStream");
        globalInterfaceImports.add("java.io.Writer");
        
        // Add some required imports for all implementations
        globalImplementationImports.add("java.io.File");
        globalImplementationImports.add("java.io.FileReader");
        globalImplementationImports.add("java.io.FileWriter");
        globalImplementationImports.add("java.io.InputStream");
        globalImplementationImports.add("java.io.InputStreamReader");
        globalImplementationImports.add("java.io.IOException");
        globalImplementationImports.add("java.io.OutputStream");
        globalImplementationImports.add("java.io.OutputStreamWriter");
        globalImplementationImports.add("java.io.Reader");
        globalImplementationImports.add("java.io.Writer");
        globalImplementationImports.add("org.xml.sax.Attributes");
        globalImplementationImports.add("org.xml.sax.EntityResolver");
        globalImplementationImports.add("org.xml.sax.ErrorHandler");
        globalImplementationImports.add("org.xml.sax.InputSource");
        globalImplementationImports.add("org.xml.sax.SAXException");
        globalImplementationImports.add("org.xml.sax.XMLReader");
        globalImplementationImports.add("org.xml.sax.helpers.DefaultHandler");
        globalImplementationImports.add("org.xml.sax.helpers.XMLReaderFactory");
    }

    /**
     * <p>
     *  This sets the output directory to output generated classes to.
     * </p>
     *
     * @param baseDir the name of the base directory where 
     *        all generated classes should be placed
     */
    public void setOutputDirectory(String outputDir) {
        if (outputDir == null) {
            throw new IllegalArgumentException("A generator cannot have a " +
                "null output directory.");
        }
        
        this.outputDir = new File(outputDir);
        uninitialize();
    }
    
    /**
     * <p>
     *  This sets the output directory to output generated classes to.
     * </p>
     *
     * @param outputDir the <code>File</code> (directory) where 
     *        all generated classes should be placed
     */
    public void setOutputDirectory(File outputDir) {
        if (outputDir == null) {
            throw new IllegalArgumentException("A generator cannot have a " +
                "null output directory.");
        }
        
        this.outputDir = outputDir;
        uninitialize();
    }

    /**
     * <p>
     *  This allows a binding to be ignored. This is an intermediary step 
     *    towards allowing binding schemas. The name of the binding should 
     *    be passed in here, and no property generation for it will occur.
     * </p>
     *
     * @param xmlName the XML local name of the binding to ignore.
     * @param xmlNamespaceURI the XML namespace URI of the binding to ignore. 
     *        The empty string indicates there is no namespace associated 
     *        with the binding.
     */
    public void setIgnoreBinding(String xmlName, String xmlNamespaceURI) {
        ignoredBindings.add(new XMLName(xmlName, xmlNamespaceURI));
    }
    
    /**
     * <p>
     *  This allows a binding to be ignored. This is an intermediary step 
     *    towards allowing binding schemas. The name of the binding should 
     *    be passed in here, and no property generation for it will occur. This
     *    convenience version supplies an empty string for the namespace URI.
     * </p>
     *
     * @param xmlName the XML local name of the binding to ignore.
     */
    public void setIgnoreBinding(String xmlName) {
        setIgnoreBinding(xmlName, "");
    }
    
    /**
     * This will determine if a particular binding should be ignored. The
     * binding is identified by its XML local name and its XML namespace URI.
     *
     * @param xmlName the XML local name of the binding.
     * @param xmlNamespaceURI the XML namespace URI associated with the binding. 
     *        An empty string indicates that there is no namespace associated 
     *        with the binding.
     *
     * @return true if and only if the binding should be ignored
     */
    protected boolean isIgnoredBinding(String xmlName, String xmlNamespaceURI) {
        if (ignoredBindings != null) {
            for (Iterator i = ignoredBindings.iterator(); i.hasNext(); ) {
                XMLName bindingName = (XMLName)i.next();
                if ((bindingName.getXMLName().equals(xmlName)) &&
                    (bindingName.getNamespaceURI().equals(xmlNamespaceURI))) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * <p>
     *  This will convert from a set of constraints to Java classes.
     *    It accesses those constraints through the
     *    <code>{@link Binding}</code> interface, which provides
     *    them in a representation-independent format, and then
     *    converts them into Java code.
     * </p>
     *
     * @param binder <code>Binder</code> with ability to convert from
     *        constraints to Zeus <code>Binding</code>s.
     * @param root whether this is the root element
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     * @throws <code>ZeusException</code> - when errors in class generation
     *         occur.
     */
    public void generate(Binding binding) 
        throws IOException, ZeusException {
            
        collapsedElements.clear();
            
        if (!isInitialized()) {
            initialize();
        }
    
        /**
         * Currently, only Containers can be generated (which
         *   includes ContainerProperties), as AtomicProperties
         *   won't result in classes.
         */
        if (binding instanceof Container) {
            Container container = (Container)binding;
            
            // Ignore Java primitive types (from collapsed elements)
            if (ClassUtils.isJavaPrimitive(binding.getJavaType())) {
                return;
            }
            
            if (binding.isXMLRootElement()) {
                /** Generate Unmarshaller class */
                generateUnmarshaller(container);
                
                /** Generate Unmarshallable interface */
                generateUnmarshallableInterface(container);
            }
        
            /** Generate interface */
            generateInterface(container);
        
            /** Generate implementation class */
            generateImplementation(container);            
        } else  {
            throw new UnsupportedBindingTypeException(binding);
        }
    }
    
    /**
     * <p>
     *  This will generate the <code>Unmarshaller</code> class for this
     *    container, which calls out to the implementation class for
     *    static unmarshalling from XML to Java.
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateUnmarshaller(Container container) 
        throws IOException {
            
        List unmarshallerImports = new LinkedList();
        
        // Get a new writer
        String filename = 
            new StringBuffer(container.getJavaType())
                .append("Unmarshaller.java")
                .toString();
        File outputFile = new File(getInterfaceDir(container), filename);
        FileWriter writer = new FileWriter(outputFile);
        
        // Write out the source file header comment
        writer.write(ZeusDefaults.GENERATED_CLASS_COMMENT);
        
        // Write the package declaration, if needed
        writePackageDeclaration(writer, container.getJavaInterfacePackage());
        
        // Write out import statements
        writeUnmarshallerImportStatements(writer, unmarshallerImports);
        
        // Write out class header
        writeUnmarshallerDeclaration(writer, container);
        
        // Write out the properties
        writeUnmarshallerProperties(writer, container);
        
        // Write out the unmarshal() methods
        writeUnmarshallerUnmarshalMethods(writer, container);
        
        writer.write("}\n");    
        writer.flush();
        writer.close();
    }
    
    /**
     * <p>
     *  This will generate the <code>Unmarshallable</code> interface for use
     *    by generated classes in the unmarshalling process.
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateUnmarshallableInterface(Container container) 
        throws IOException {
            
        // Get a new writer
        String filename = 
            new StringBuffer("Unmarshallable.java")
                .toString();
        File outputFile = new File(getInterfaceDir(container), filename);
        FileWriter writer = new FileWriter(outputFile);
        
        // Write out the source file header comment
        writer.write(ZeusDefaults.GENERATED_CLASS_COMMENT);
        
        // Write the package declaration, if needed
        writePackageDeclaration(writer, container.getJavaInterfacePackage());
        
        // Write out import statements
        writer.write("import org.xml.sax.ContentHandler;\n\n");
        
        // Write out interface declaration
        writer.write("public interface Unmarshallable ");
        writer.write("extends ContentHandler {\n\n");
            
        // Write out additional methods
        writer.write("    public Unmarshallable getCurrentUNode();\n\n");
        writer.write("    public void setCurrentUNode(");
        writer.write("Unmarshallable currentUNode);\n\n");
        writer.write("    public Unmarshallable getParentUNode();\n\n");
        writer.write("    public void setParentUNode(");
        writer.write("Unmarshallable parentUNode);\n\n");
        
        writer.write("}\n");    
        writer.flush();
        writer.close();
    }
    
    /**
     * <p>
     *  This will generate the interface (or interfaces, through recursion)
     *    for the supplied <code>Container</code>.
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateInterface(Container container) 
        throws IOException {
            
        List interfaceImports = new LinkedList();
        
        // Get a new writer
        String filename = 
            new StringBuffer(container.getJavaType())
                .append(".java")
                .toString();
        File outputFile = new File(getInterfaceDir(container), filename);
        FileWriter writer = new FileWriter(outputFile);
        
        // Write out the source file header comment
        writer.write(ZeusDefaults.GENERATED_CLASS_COMMENT);
        
        // Write the package declaration, if needed
        writePackageDeclaration(writer, container.getJavaInterfacePackage());
        
        /**
         * If any of the properties do not have a Java package then we
         *   must import them, unless they are primitive types.
         */
        List properties = container.getProperties();
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if (property.getJavaInterfacePackage().equals("")) {
                if (!ClassUtils.isJavaPrimitive(property.getJavaType())) {
                    interfaceImports.add(property.getJavaType());
                }
            }
            
            if (property.isCollection()) {
                String collectionClass = property.getJavaCollectionClass();
                if (!interfaceImports.contains(collectionClass)) {
                    interfaceImports.add(collectionClass);
                }
            }
        }
        
        // Add Serializable import if needed
        if (container.isJavaSerializable()) {
            interfaceImports.add("java.io.Serializable");
        }
        
        // Write out import statements
        writeInterfaceImportStatements(writer, interfaceImports);
        
        // Write out class header
        writeInterfaceDeclaration(writer, container);

        // Write out element and attribute ordering
        writeElementAndAttributeArrays(writer, container);

        // Write out method declarations
        writeInterfaceMethods(writer, container.getProperties());        
        
        // Write out method declarations for marshalling to XML
        writeInterfaceMarshalMethods(writer);

        writer.write("}\n");    
        writer.flush();
        writer.close();
    }

    /**
     * <p>
     *  This will generate the implementation (or implemenations, through
     *    recursion) for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateImplementation(Container container) 
        throws IOException {
            
        List implementationImports = new LinkedList();
        
        // Get a new File handle
        String filename = container.getJavaType() + "Impl.java";
        File outputFile = new File(getImplementationDir(container), filename);
        FileWriter writer = new FileWriter(outputFile);
        
        // Write out the source file header comment
        writer.write(ZeusDefaults.GENERATED_CLASS_COMMENT);

        // Add a package statement if necessary.
        writePackageDeclaration(writer, 
                                container.getJavaImplementationPackage());

        /**
         * If the interface is in the default namespace then we have to
         *   import it. So we might as well anyway.
         */
        if (container.getJavaInterfacePackage() != 
            container.getJavaImplementationPackage()) {
                
            StringBuffer containerPackage = new StringBuffer();
            if (!container.getJavaInterfacePackage().equals("")) {
                containerPackage.append(container.getJavaInterfacePackage())
                                .append(".");
            }
            containerPackage.append(container.getJavaType());
            implementationImports.add(containerPackage.toString());
        }

        /**
         * If any of the properties do not have a Java package then we
         *   must import them, unless they are primitive types.
         */
        boolean firstCollection = true;
        List properties = container.getProperties();
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if (property.getJavaInterfacePackage().equals("")) {
                if (!ClassUtils.isJavaPrimitive(property.getJavaType())) {
                    implementationImports.add(property.getJavaType());
                }
            }
            
            if (property.isCollection()) {
                if (firstCollection) {
                    implementationImports.add("java.util.Iterator");
                    firstCollection = false;
                }
                String collectionClass = property.getJavaCollectionClass();
                if (!implementationImports.contains(collectionClass)) {
                    implementationImports.add(collectionClass);
                    implementationImports.add(ClassUtils.getCollectionImplClass(
                        collectionClass));
                }
            }
        }

        // Write out import statements
        writeImplementationImportStatements(writer, implementationImports);

        // Write out the implementation declaration
        writeImplementationDeclaration(writer, container.getJavaType());
        
        // Write out each property
        writeImplementationProperties(writer, properties);
        
        // Write out unmarshaller properties
        writeImplementationUnmarshallerProperties(writer, container);
        
        // Write class constructor
        writeImplementationConstructor(writer, 
                                       container.getJavaType(), 
                                       properties);       
        
        // Write out each method implementation
        writeImplementationMethods(writer, container);
        
        // Write out the implementation of the marshal() methods
        writeImplementationMarshalMethods(writer, container);
        
        // Write out the implementation of the unmarshal() methods
        writeImplementationUnmarshalMethods(writer, container);
        
        // Write out SAX Handler methods
        writeImplementationSAXHandlerMethods(writer, container);
        
        writer.write("}\n");
        
        writer.flush();
        writer.close();
    }
    
    /**
     * <p>
     *  This writes out the package declaration for this class, if needed.
     * </p>
     *
     * @param writer <code>Writer</code> to perform output to
     * @param packageName the package of this class
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writePackageDeclaration(Writer writer, String packageName) 
        throws IOException {
            
        // Generate a package statement if necessary.
        if (!packageName.equals("")) {
            writer.write("package");
            writer.write(" ");
            writer.write(packageName);
            writer.write(";\n\n");
        }
    }
    
    /**
     * <p>
     *  This will take the supplied list of class names, and add import
     *    statements for each of the classes in the list. It will also write
     *    out all global import statements for unmarshaller classes.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param importClasses <code>List</code> of classes to add imports for
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeUnmarshallerImportStatements(Writer writer, 
                                                     List importClasses) 
        throws IOException {

        // Write out global imports
        if (globalUnmarshallerImports.size() > 0) {
            writer.write("// Global Unmarshaller Import Statements\n");
        }
        for (Iterator i = globalUnmarshallerImports.iterator(); i.hasNext(); ) {
            writer.write("import ");
            writer.write((String)i.next());
            writer.write(";\n");
        }
        if (globalUnmarshallerImports.size() > 0) {
            writer.write("\n");
        }
        
        // Write out local imports
        if (importClasses.size() > 0) {
            writer.write("// Local Unmarshaller Import Statements\n");
        }
        for (Iterator i = importClasses.iterator(); i.hasNext(); ) {
            writer.write("import ");
            writer.write((String)i.next());
            writer.write(";\n");
        }
        if (importClasses.size() > 0) {
            writer.write("\n");
        }
    }
    
    /**
     * <p>
     *  This writes out the unmarshaller declaration (the class header).
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container the container for the class
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeUnmarshallerDeclaration(Writer writer, 
                                                Container container)
        throws IOException {
            
        writer.write("public class ");
        writer.write(container.getJavaType());
        writer.write("Unmarshaller {\n\n");
    }
    
    /**
     * <p>
     *  This will write out the unmarshaller properties.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container the container to write the unmarshalling for.
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeUnmarshallerProperties(Writer writer,
                                               Container container)
        throws IOException {
            
        writer.write("    /** The EntityResolver for parser resolution. */\n");
        writer.write("    private static EntityResolver entityResolver;\n\n");
        
        writer.write("    /** The ErrorHandler for parser resolution. */\n");
        writer.write("    private static ErrorHandler errorHandler;\n\n");
    }
    
    /**
     * <p>
     *  This will write out the unmarshaller version of the various 
     *    <code>unmarshal</code> methods supported by generated classes.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container the container to write the unmarshalling for.
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeUnmarshallerUnmarshalMethods(Writer writer,
                                                     Container container)
        throws IOException {

        String className = container.getJavaType();
            
        // Set an entity resolver
        writer.write("    /**\n");
        writer.write("     * <p>\n");
        writer.write("     *  This sets a SAX <code>EntityResolver</code> for");
        writer.write(" this unmarshalling process.\n");
        writer.write("     * </p>\n");
        writer.write("     *\n");
        writer.write("     * @param resolver the entity resolver ");
        writer.write("to use.\n");
        writer.write("     */\n");
        writer.write("    public static void setEntityResolver(");
        writer.write("EntityResolver resolver) {\n");
        writer.write("        entityResolver = resolver;\n");
        writer.write("    }\n\n");
        
        // Set an error handler
        writer.write("    /**\n");
        writer.write("     * <p>\n");
        writer.write("     *  This sets a SAX <code>ErrorHandler</code> for");
        writer.write(" this unmarshalling process.\n");
        writer.write("     * </p>\n");
        writer.write("     *\n");
        writer.write("     * @param handler the error handler ");
        writer.write("to use.\n");
        writer.write("     */\n");
        writer.write("    public static void setErrorHandler(");
        writer.write("ErrorHandler handler) {\n");
        writer.write("        errorHandler = handler;\n");
        writer.write("    }\n\n");
            
        // unmarshal(File file)
        writer.write("    public static ");
        writer.write(className);
        writer.write(" unmarshal(File file)");
        writer.write(" throws IOException {\n");
        writer.write("        // Delegate to the unmarshal(Reader) method\n");
        writer.write("        return unmarshal(new FileReader(file));\n");
        writer.write("    }\n\n");
        
        // unmarshal(File file, boolean validate)
        writer.write("    public static ");
        writer.write(className);
        writer.write(" unmarshal(File file, boolean validate)");
        writer.write(" throws IOException {\n");
        writer.write("        // Delegate to the unmarshal(Reader) method\n");
        writer.write("        return unmarshal(new FileReader(file), ");
        writer.write("validate);\n");
        writer.write("    }\n\n");
            
        // unmarshal(InputStream inputStream)
        writer.write("    public static ");
        writer.write(className);
        writer.write(" unmarshal(InputStream inputStream)");
        writer.write(" throws IOException {\n");
        writer.write("        // Delegate to the unmarshal(Reader) method\n");
        writer.write("        ");
        writer.write("return unmarshal(new InputStreamReader(inputStream));\n");
        writer.write("    }\n\n");
        
        // unmarshal(InputStream inputStream, boolean validate)
        writer.write("    public static ");
        writer.write(className);
        writer.write(" unmarshal(InputStream inputStream, boolean validate)");
        writer.write(" throws IOException {\n");
        writer.write("        // Delegate to the unmarshal(Reader) method\n");
        writer.write("        ");
        writer.write("return unmarshal(new InputStreamReader(inputStream), ");
        writer.write("validate);\n");
        writer.write("    }\n\n");
        
        // unmarshal(Reader reader)
        writer.write("    public static ");
        writer.write(className);
        writer.write(" unmarshal(Reader reader)");
        writer.write(" throws IOException {\n");
        writer.write("        // Delegate with default validation state\n");
        writer.write("        ");
        writer.write("return unmarshal(reader, ");
        writer.write(ZeusDefaults.DEFAULT_VALIDATION_STATE.toString());
        writer.write(");\n");
        writer.write("    }\n\n");
        
        // unmarshal(Reader reader, boolean validate)
        writer.write("    public static ");
        writer.write(className);
        writer.write(" unmarshal(Reader reader, boolean validate)");
        writer.write(" throws IOException {\n");
        writer.write("        // Set the entity resolver, if needed\n");
        writer.write("        if (entityResolver != null) {\n");
        writer.write("            ");
        writer.write(className);
        writer.write("Impl.setEntityResolver(entityResolver);\n");
        writer.write("        }\n\n");
        writer.write("        // Set the error handler, if needed\n");
        writer.write("        if (errorHandler != null) {\n");
        writer.write("            ");
        writer.write(className);
        writer.write("Impl.setErrorHandler(errorHandler);\n");
        writer.write("        }\n\n");
        writer.write("        // Unmarshal using the implementation class\n");
        writer.write("        return ");
        writer.write(className);
        writer.write("Impl.unmarshal(reader, validate);\n");
        writer.write("    }\n\n");
    }
    
    /**
     * <p>
     *  This will take the supplied list of class names, and add import
     *    statements for each of the classes in the list. It will also write
     *    out all global import statements for interface classes.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param importClasses <code>List</code> of classes to add imports for
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeInterfaceImportStatements(Writer writer, 
                                                  List importClasses) 
        throws IOException {

        // Write out global imports
        if (globalInterfaceImports.size() > 0) {
            writer.write("// Global Interface Import Statements\n");
        }
        for (Iterator i = globalInterfaceImports.iterator(); i.hasNext(); ) {
            writer.write("import ");
            writer.write((String)i.next());
            writer.write(";\n");
        }
        if (globalInterfaceImports.size() > 0) {
            writer.write("\n");
        }
        
        // Write out local imports
        if (importClasses.size() > 0) {
            writer.write("// Local Interface Import Statements\n");
        }
        for (Iterator i = importClasses.iterator(); i.hasNext(); ) {
            writer.write("import ");
            writer.write((String)i.next());
            writer.write(";\n");
        }
        if (importClasses.size() > 0) {
            writer.write("\n");
        }
    }
    
    /**
     * <p>
     *  This generates an array of element and attribute names, for
     *    correct ordering in marshalling.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container <code>Container</code> with properties 
     *        to generate
     * @throws <code>IOException</code> when output errors occur.
     */
    private void writeElementAndAttributeArrays(Writer writer,
                                                Container container) 
        throws IOException {

        // Initialize variables
        List properties = container.getProperties();
        StringBuffer attributeNames = new StringBuffer();
        boolean firstAtt = true;
        StringBuffer elementNames = new StringBuffer();
        boolean firstElem = true;        
        BitSet modifiers = null;
        
        // construct the contents of the arrays 
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            modifiers = property.getModifier();

            // See if it's an attribute            
            if (modifiers.get(Property.SOURCE_ATTLIST)) {
                // append the comma separator
                if (firstAtt) {
                    firstAtt = false;
                } else {
                    attributeNames.append(", ");
                }
                    
                // append the name of the attribute
                attributeNames.append("\"");
                attributeNames.append(property.getXMLName());
                attributeNames.append("\"");

                // See if it's an element
            } else if (modifiers.get(Property.SOURCE_ELEMENT)) {
                // append the comma separator
                if (firstElem) {
                    firstElem = false;
                } else {
                    elementNames.append(", ");
                }
                    
                // append the name of the element
                elementNames.append("\"");
                elementNames.append(property.getXMLName());
                elementNames.append("\"");
            }
        }

        writer.write("    public static final String ");
        writer.write(ZeusDefaults.XML_NAME_FIELD);
        writer.write(" = \"");
        writer.write(container.getXMLName());
        writer.write("\";\n");

        writer.write("    public static final String[] ");
        writer.write(ZeusDefaults.ATTRIBUTE_ARRAY_FIELD);
        writer.write(" = {");
        writer.write(attributeNames.toString());
        writer.write("};\n");
        
        writer.write("    public static final String[] ");
        writer.write(ZeusDefaults.ELEMENT_ARRAY_FIELD);
        writer.write(" = {");
        writer.write(elementNames.toString());
        writer.write("};\n\n");
    }
    
    /**
     * <p>
     *  This writes out the interface declaration (the class header).
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container the container to write the interface for.
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeInterfaceDeclaration(Writer writer, 
                                             Container container)
        throws IOException {
            
        writer.write("public interface ");
        writer.write(container.getJavaType());

        // Make serializable if needed
        if (container.isJavaSerializable()) {
            writer.write(" extends Serializable");
        }
        writer.write(" {\n\n");
    }
    
    /**
     * <p>
     *  This writes out the various method declarations for an interface.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param properties <code>List</code> of properties to write out method
     *        declarations for
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeInterfaceMethods(Writer writer, List properties)
        throws IOException {
        
        // Write out each method        
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if (isIgnoredBinding(property.getXMLName(),
                                 property.getXMLNamespaceURI()) ||
                (property.getXMLName().startsWith("xmlns"))) {
                continue;
            }
            writePropertyMethodDecl(writer, property);
        }
    }
    
    /**
     * <p>
     *  This will take a <code>Property</code> and generate the code
     *    for that property as an accessor/mutator method (definition only).
     * </p>
     *
     * @param writer <code>Writer</code> to write method declarations to.
     * @param property <code>Property</code> to generate methods for.
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     */       
    protected void writePropertyMethodDecl(Writer writer, Property property) 
        throws IOException {
        
        // See if this is a Collection or not
        boolean isCollection = property.isCollection();
        String javaType = property.getJavaType();
        String collectionClass = 
            NamingUtils.removePackage(property.getJavaCollectionClass());

        /**
         * There is a conditional here, although all that ever should
         *   get to here are AtomicProperty objects. But just in case...
         */
        if (property instanceof AtomicProperty) {
            AtomicProperty atomicProperty = (AtomicProperty)property;
            
            // Write out the property accessor
            writer.write("    public ");
            if (isCollection) {
                writer.write(collectionClass);
            } else {
                writer.write(javaType);
            }
            writer.write(" get");
            writer.write(CapitalizationUtils.initialUpper(
                atomicProperty.getJavaName()));
            if (isCollection) {
                writer.write("List");
            }            
            writer.write("();\n\n");
            
            /**
             * Write out the property mutator if the property is not #FIXED
             *   #FIXED attributes cannot be changed...
             */
            if (!atomicProperty.getModifier().get(Property.MUTABILITY_FINAL)) {
                writer.write("    public void set");
                writer.write(CapitalizationUtils.initialUpper(
                    atomicProperty.getJavaName()));
                if (isCollection) {
                    writer.write("List");
                }            
                writer.write("(");
                if (isCollection) {
                    writer.write(collectionClass);
                } else {
                    writer.write(javaType);
                }            
                writer.write(" ");
                if (isCollection) {
                    writer.write(atomicProperty.getJavaName());
                    writer.write("List");
                } else {
                    writer.write(atomicProperty.getJavaVariableName());
                }
               
                /**
                 * If the property has an enumerator, we want to throw 
                 *   an IllegalArgumentException if the incoming data on 
                 *   set operations doesn't pass the validation
                 */               
                if (atomicProperty.hasEnumeration()) {
                    writer.write(")\n       " +
                        "throws IllegalArgumentException;\n");
                } else {
                    writer.write(");\n\n");
                }
               
                // If this is a Collection, add the "add" method
                if (isCollection) {
                    writer.write("    public void add");
                    writer.write(CapitalizationUtils.initialUpper(
                        atomicProperty.getJavaName()));
                    writer.write("(");
                    writer.write(javaType);
                    writer.write(" ");
                    writer.write(atomicProperty.getJavaVariableName());
                    writer.write(");\n\n");

                    // add the "remove" method                   
                    writer.write("    public void remove");
                    writer.write(CapitalizationUtils.initialUpper(
                        atomicProperty.getJavaName()));
                    writer.write("(");
                    writer.write(javaType);
                    writer.write(" ");
                    writer.write(atomicProperty.getJavaVariableName());
                    writer.write(");\n\n");
                }
            }               
        }
    }
    
    /**
     * <p>
     *  This will write out the declaration of the various <code>marshal</code>
     *    methods supported by generated classes.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeInterfaceMarshalMethods(Writer writer)
        throws IOException {

        // marshal(File file)
        writer.write("    public void marshal(File file)");
        writer.write(" throws IOException;\n\n");
            
        // marshal(OutputStream outputStream)
        writer.write("    public void marshal(OutputStream outputStream)");
        writer.write(" throws IOException;\n\n");
        
        // marshal(Writer writer)
        writer.write("    public void marshal(Writer writer)");
        writer.write(" throws IOException;\n\n");
    }
    
    /**
     * <p>
     *  This will take the supplied list of class names, and add import
     *    statements for each of the classes in the list. It will also write
     *    out all global import statements for implementation classes.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param importClasses <code>List</code> of classes to add imports for
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplementationImportStatements(Writer writer, 
                                                       List importClasses) 
        throws IOException {

        // Write out global imports
        if (globalImplementationImports.size() > 0) {
            writer.write("// Global Implementation Import Statements\n");
        }
        for (Iterator i = globalImplementationImports.iterator(); 
             i.hasNext(); ) {
                 
            writer.write("import ");
            writer.write((String)i.next());
            writer.write(";\n");
        }
        if (globalImplementationImports.size() > 0) {
            writer.write("\n");
        }
        
        // Write out local imports
        if (importClasses.size() > 0) {
            writer.write("// Local Implementation Import Statements\n");
        }
        for (Iterator i = importClasses.iterator(); i.hasNext(); ) {
            writer.write("import ");
            writer.write((String)i.next());
            writer.write(";\n");
        }
        if (importClasses.size() > 0) {
            writer.write("\n");
        }
    }
    
    /**
     * <p>
     *  This writes out the implementation declaration (the class header).
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param interfaceName the name of the interface class being implemented
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplementationDeclaration(Writer writer, 
                                                  String interfaceName)
        throws IOException {
            
        writer.write("public class ");
        writer.write(interfaceName);
        writer.write("Impl extends DefaultHandler implements Unmarshallable, ");
        writer.write(interfaceName);
        writer.write(" {\n\n");
    }
    
    /**
     * <p>
     *  This writes out the properties for an implementation as variables.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param properties <code>List</code> of properties to write out as
     *        member variables
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplementationProperties(Writer writer, List properties)
        throws IOException {
            
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if (isIgnoredBinding(property.getXMLName(),
                                 property.getXMLNamespaceURI()) ||
                (property.getXMLName().startsWith("xmlns"))) {
                continue;
            }
            writeProperty(writer, property);
        }
        
        writer.write("\n");
    }
    
    /**
     * <p>
     *  This will take a <code>Property</code> and generate the code
     *    for that property as a member variable.
     * </p>
     *
     * @param writer <code>Writer</code> to write variable declaration to.
     * @param property <code>Property</code> to generate code for.
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     */    
    protected void writeProperty(Writer writer, Property property) 
        throws IOException {
            
        // See if this is a Collection or not
        String javaType = property.getJavaType();
        boolean isCollection = property.isCollection();                        
        
        /**
         * There is a conditional here, although all that ever should
         *   get to here are AtomicProperty objects. But just in case...
         */
        if (property instanceof AtomicProperty) {            
            AtomicProperty atomicProperty = (AtomicProperty)property;
            // Write out the property
            writer.write("    ");
            writer.write(atomicProperty.getModifierString());
            writer.write(" ");
            if (isCollection) {
                writer.write(NamingUtils.removePackage(
                    atomicProperty.getJavaCollectionClass()));
            } else {
                writer.write(atomicProperty.getJavaType());
            }
            writer.write(" ");

            if (isCollection) {
                writer.write(atomicProperty.getJavaName());
                writer.write("List");
            } else {
                writer.write(atomicProperty.getJavaVariableName());
            }
            writer.write(";\n");
            
            /*
             * If the property has an enumeration, generate variables to
             *   hold the data to validate incoming data on set operations
             */            
            if (atomicProperty.hasEnumeration()) {
                    String myName = atomicProperty.getJavaVariableName();
                    writer.write("    private static boolean ");
                    writer.write(ZeusDefaults.ZEUS_INTERNAL_PREFIX);
                    writer.write(myName);
                    writer.write("Initialized;\n");
                    writer.write("    private static java.util.Vector ");
                    writer.write(ZeusDefaults.ZEUS_INTERNAL_PREFIX);
                    writer.write("valid");
                    writer.write(myName);
                    writer.write(";\n");
            }

            /**
             * If the property is an attribute and has a primitive type then 
             *   we must add a corresponding boolean to determine if the 
             *   value has been set.
             */
            if ((atomicProperty.getJavaInterfacePackage().length() == 0) &&
                ClassUtils.isJavaPrimitive(atomicProperty.getJavaType())) {
                writer.write("    private boolean ");
                writer.write(ZeusDefaults.ZEUS_INTERNAL_PREFIX);
                writer.write(atomicProperty.getJavaName());
                writer.write("Set;\n");
            }
        }
    }
    
    /**
     * <p>
     *  This will write out properties required for unmarshalling from XML
     *    into this Java class.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container the container writing out for
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplementationUnmarshallerProperties(
        Writer writer, Container container) throws IOException {
            
        writer.write("    /** The current node in unmarshalling */\n");
        writer.write("    private Unmarshallable zeus_currentUNode;\n\n");
        
        writer.write("    /** The parent node in unmarshalling */\n");
        writer.write("    private Unmarshallable zeus_parentUNode;\n\n");
        
        writer.write("    /** Whether this node has been handled */\n");
        writer.write("    private boolean zeus_thisNodeHandled = false;\n\n");
        
        writer.write("    /** The EntityResolver for SAX parsing to use */\n");
        writer.write("    private static EntityResolver entityResolver;\n\n");
        
        writer.write("    /** The ErrorHandler for SAX parsing to use */\n");
        writer.write("    private static ErrorHandler errorHandler;\n\n");
    }
                                                             
    
    /**
     * <p>
     *  This writes out the constructor for an implementation class
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param className the name of the implementation class
     * @param properties <code>List</code> of properties to write out as
     *        member variables
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplementationConstructor(Writer writer, 
                                                  String className,
                                                  List properties)
        throws IOException {
            
        writer.write("    public ");
        writer.write(className);
        writer.write("Impl() {\n");

        // Initialize any List variables needed
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();   
            if (isIgnoredBinding(property.getXMLName(),
                                 property.getXMLNamespaceURI()) ||
                (property.getXMLName().startsWith("xmlns"))) {
                continue;
            }

            // Initialize collection classes
            if (property.isCollection()) {
                writer.write("        ");
                writer.write(property.getJavaName());
                writer.write("List = new ");
                writer.write(NamingUtils.removePackage(
                    ClassUtils.getCollectionImplClass(
                        property.getJavaCollectionClass())));
                writer.write("();\n");
            }
            
            // Handle default values
            if (property.hasDefaultValue()) {
                writer.write("        ");
                writer.write(property.getJavaName());
                writer.write(" = ");
                
                if ((property.getJavaInterfacePackage().length() == 0) &&
                    (ClassUtils.isJavaPrimitive(property.getJavaType()))) {
                    // in this case the property's type is a primitive
                    generatePrimitiveAssignment(
                        writer, property,
                        "\"" + (String)property.getDefaultValue() + "\"");
                } else {
                    
                    /**
                     * In this case the property's type is not a String but
                     * is also not a primitive
                     */
                    writer.write("new ");
                    String pkg = property.getJavaInterfacePackage();
                    if (pkg.length() != 0) {
                        writer.write(pkg);
                        writer.write(".");
                    }
                    writer.write(property.getJavaType());
                    writer.write("(\"");
                    writer.write((String)property.getDefaultValue());
                    writer.write("\")");
                }
                
                writer.write(";\n");
            }
            
            /**
             * If the property has an enumeration, generate variables to
             *   hold the data to validate incoming data on set operations
             */            
            if (property.hasEnumeration()) {
                    String myName = property.getJavaVariableName();
                    writer.write("        ");
                    writer.write(ZeusDefaults.ZEUS_INTERNAL_PREFIX);
                    writer.write(myName);
                    writer.write("Initialized = false;\n");
                    writer.write("        ");
                    writer.write(ZeusDefaults.ZEUS_INTERNAL_PREFIX);
                    writer.write("valid");
                    writer.write(myName);
                    writer.write(" = null;\n");
            }

            /**
             * If the property is an attribute and has a primitive type then 
             *   we must add a corresponding boolean to determine if the 
             *   value has been set.
             */
            if ((property.getJavaInterfacePackage().length() == 0) &&
                (ClassUtils.isJavaPrimitive(property.getJavaType()))) {
                
                writer.write("        ");
                writer.write(ZeusDefaults.ZEUS_INTERNAL_PREFIX);
                writer.write(property.getJavaName());
                writer.write("Set = false;\n");
            }
        }
        writer.write("    }\n\n"); 
    }
    
    
    /**
     * <p>
     *  This writes out the properties for an implementation as methods,
     *    including accessors, mutators, and collection-style methods if
     *    needed.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container <code>Container</code> with properties to generate
     *        methods for
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplementationMethods(Writer writer, 
                                              Container container)
        throws IOException {
            
        List properties = container.getProperties();
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if (isIgnoredBinding(property.getXMLName(),
                                 property.getXMLNamespaceURI()) ||
                (property.getXMLName().startsWith("xmlns"))) {
                continue;
            }
            writePropertyMethodImplementation(writer, property);
        }
    }
   
    /**
     * <p>
     *  This will take a <code>Property</code> and generate the code
     *    for that property as an accessor/mutator method (with implementation).
     * </p>
     *
     * @param writer <code>Writer</code> to write method implementations to.
     * @param property <code>Property</code> to generate code for.
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     */       
    protected void writePropertyMethodImplementation(Writer writer, 
                                                     Property property) 
        throws IOException {
        
        // See if this is a Collection or not
        boolean isCollection = property.isCollection();
        String javaType = property.getJavaType();
        
        /**
         * There is a conditional here, although all that ever should
         *   get to here are AtomicProperty objects. But just in case...
         */
        if (property instanceof AtomicProperty) {
            AtomicProperty atomicProperty = (AtomicProperty)property;
            
            // Write out the property accessor
            writer.write("    public ");
            if (isCollection) {
                writer.write(NamingUtils.removePackage(
                    atomicProperty.getJavaCollectionClass()));
            } else {
                writer.write(javaType);
            }            
            writer.write(" get");
            writer.write(CapitalizationUtils
                         .initialUpper(atomicProperty.getJavaName()));
            if (isCollection) {
                writer.write("List");
            }
            writer.write("() {\n");
            writer.write("        return ");

            if (isCollection) {
                writer.write(atomicProperty.getJavaName());
                writer.write("List");
            } else {
                writer.write(atomicProperty.getJavaVariableName());
            }
            writer.write(";\n");    
            writer.write("    }\n\n");
            
            // Write out the property mutator if the property is not #FIXED
            if (!atomicProperty.getModifier().get(Property.MUTABILITY_FINAL)) {
                writer.write("    public void set");
                writer.write(CapitalizationUtils.initialUpper(
                    atomicProperty.getJavaName()));
                if (isCollection) {
                    writer.write("List");
                }            
                writer.write("(");
                if (isCollection) {
                    writer.write(NamingUtils.removePackage(
                        atomicProperty.getJavaCollectionClass()));
                } else {
                    writer.write(javaType);
                }            
                writer.write(" ");
                if (isCollection) {
                    writer.write(atomicProperty.getJavaName());
                    writer.write("List");
                } else {
                    writer.write(atomicProperty.getJavaVariableName());
                }

                /**
                 * If the property has an enumeration, then generate 
                 *   the code to initialize the validation data and to 
                 *   validate incoming data
                 */
                if (atomicProperty.hasEnumeration()) {
                    String myName = atomicProperty.getJavaVariableName();
                    String flagName = 
                        new StringBuffer(ZeusDefaults.ZEUS_INTERNAL_PREFIX)
                            .append(myName)
                            .append("Initialized")
                            .toString();
                    String vectorName = 
                        new StringBuffer(ZeusDefaults.ZEUS_INTERNAL_PREFIX)
                            .append("valid")
                            .append(myName)
                            .toString();
                    Vector theEnumeration = atomicProperty.getEnumeration();

                    writer.write(")\n        ");
                    writer.write("throws IllegalArgumentException {\n");
                    writer.write("        if (!");
                    writer.write(flagName);
                    writer.write(") {\n");
                    writer.write("            ");
                    writer.write(vectorName);
                    writer.write(" = new java.util.Vector();\n");
                    for (int i=0, size=theEnumeration.size(); i < size; i++) {
                        writer.write("            ");
                        writer.write(vectorName);
                        writer.write(".addElement(\"");
                        writer.write((String)theEnumeration.elementAt(i));
                        writer.write("\");\n");
                    }
                    writer.write("            ");
                    writer.write(flagName);
                    writer.write(" = true;\n");
                    writer.write("        }\n");
                    writer.write("        if (!");
                    writer.write(vectorName);
                    writer.write(".contains(");
                    writer.write(myName);
                    writer.write(")) {\n");
                    writer.write("              throw new ");
                    writer.write("IllegalArgumentException(\"");
                    writer.write("Illegal value for attribute '");
                    writer.write(myName);
                    writer.write("'\");\n");
                    writer.write("        } else {\n");
                    writer.write("              this.");
                    writer.write(myName);
                    writer.write(" = ");
                    writer.write(myName);
                    writer.write(";\n");
                    writer.write("            this.");
                    writer.write(ZeusDefaults.ZEUS_INTERNAL_PREFIX);
                    writer.write(myName);
                    writer.write("Set = true;\n");
                    writer.write("        }\n");
                } else {    
                    writer.write(") {\n");                
                    writer.write("        this.");

                    if (isCollection) {
                        writer.write(atomicProperty.getJavaName());
                        writer.write("List");
                    } else {
                        writer.write(atomicProperty.getJavaVariableName());
                    }

                    writer.write(" = ");

                    if (isCollection) {
                        writer.write(atomicProperty.getJavaName());
                        writer.write("List");
                    } else {
                        writer.write(atomicProperty.getJavaVariableName());
                    }
                    writer.write(";\n");

                    /**
                     * If this property has a primitive type then mark that it
                     *   has been set.
                     */
                    if ((atomicProperty.getJavaInterfacePackage()
                                       .length() == 0) &&
                        (ClassUtils.isJavaPrimitive(
                            atomicProperty.getJavaType()))) {
                                
                        writer.write("        ");
                        writer.write(ZeusDefaults.ZEUS_INTERNAL_PREFIX);
                        writer.write(atomicProperty.getJavaName());
                        writer.write("Set = true;\n");
                    }
                }                   
                writer.write("    }\n\n");            
            
                // If this is a Collection, add the "add" and "remove" methods
                if (isCollection) {
                    // Handle the "add" method
                    writer.write("    public void add");
                    writer.write(CapitalizationUtils.initialUpper(
                        atomicProperty.getJavaName()));
                    writer.write("(");
                    writer.write(javaType);
                    writer.write(" ");
                    writer.write(atomicProperty.getJavaName());
                    writer.write(") {\n");
                    writer.write("        ");
                    writer.write(atomicProperty.getJavaName());
                    writer.write("List.add(");
                    writer.write(atomicProperty.getJavaName());
                    writer.write(");\n");
                    writer.write("    }\n\n");
                    
                    // add the "remove" method
                    writer.write("    public void remove");
                    writer.write(CapitalizationUtils.initialUpper(
                        atomicProperty.getJavaName()));
                    writer.write("(");
                    writer.write(javaType);
                    writer.write(" ");
                    writer.write(atomicProperty.getJavaName());
                    writer.write(") {\n");
                    writer.write("        ");
                    writer.write(atomicProperty.getJavaName());
                    writer.write("List.remove(");
                    writer.write(atomicProperty.getJavaName());
                    writer.write(");\n");
                    writer.write("    }\n\n");
                }
            }
        }
    }
    
    /**
     * <p>
     *  This will write out the implementation of the various 
     *    <code>marshal</code>. methods supported by generated classes.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplementationMarshalMethods(Writer writer,
                                                     Container container)
        throws IOException {

        // marshal(File file)
        writer.write("    public void marshal(File file)");
        writer.write(" throws IOException {\n");
        writer.write("        // Delegate to the marshal(Writer) method\n");
        writer.write("        marshal(new FileWriter(file));\n");
        writer.write("    }\n\n");
            
        // marshal(OutputStream outputStream)
        writer.write("    public void marshal(OutputStream outputStream)");
        writer.write(" throws IOException {\n");
        writer.write("        // Delegate to the marshal(Writer) method\n");
        writer.write("        ");
        writer.write("marshal(new OutputStreamWriter(outputStream));\n");
        writer.write("    }\n\n");
        
        // marshal(Writer writer)
        writer.write("    public void marshal(Writer writer)");
        writer.write(" throws IOException {\n");
        writer.write("        // Write out the XML declaration\n");
        writer.write("        writer.write(\"<?xml version=\\\"1.0\\\" ");
        writer.write("encoding=\\\"UTF-8\\\"?>\\n\\n\");\n\n");
        writer.write("        // Now start the recursive writing\n");
        writer.write("        writeXMLRepresentation(writer, \"\");\n\n");
        writer.write("        // Close up\n");
        writer.write("        writer.flush();\n");
        writer.write("        writer.close();\n");
        writer.write("    }\n\n");
        
        writer.write("    ");
        writer.write("protected void writeXMLRepresentation(Writer writer,\n");
        writer.write("    ");
        writer.write("                                      String indent)\n");
        writer.write("        throws IOException {\n\n");
            
        // Write out this element
        writer.write("        writer.write(indent);\n");
        writer.write("        writer.write(\"<");
        writer.write(container.getXMLName());
        writer.write("\");\n\n");
        
        // Write in attributes
        writer.write("        // Handle attributes (if needed)\n");
        
        // Iterate through this container's child properties
        List properties = container.getProperties();
        List childElements = new LinkedList();
        boolean hasChildren = false;
        String xmlValueVariable = null;
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            String xmlName = property.getXMLName();
            BitSet modifiers = property.getModifier();
            
            if (modifiers.get(Property.SOURCE_ATTLIST)) {
                writer.write("        if (");
                writer.write(ZeusDefaults.ZEUS_INTERNAL_PREFIX);
                writer.write(property.getJavaName());
                writer.write("Set) {\n");
                writer.write("            writer.write(\" ");
                writer.write(property.getXMLName());
                writer.write("=\\\"\");\n");
                writer.write("            writer.write(escapeAttributeValue(");
                writer.write(property.getJavaVariableName());
                writer.write("));\n");
                writer.write("            writer.write(\"\\\"\");\n");
                writer.write("        }\n");
            } else if (modifiers.get(Property.SOURCE_ELEMENT)) {
                hasChildren = true;
                childElements.add(property);
            } else if (xmlName.equals(ZeusDefaults.PCDATA_XML_NAME)) {
                xmlValueVariable = property.getJavaName();
            }
        }
        writer.write("        writer.write(\">\");\n");
        
        // Handle elements
        if (hasChildren) {
            writer.write("        writer.write(\"\\n\");\n\n");
            writer.write("        // Handle child elements\n");
        }
        for (Iterator i = childElements.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if (property.isCollection()) {
                writer.write("        for (Iterator i=");
                writer.write(property.getJavaName());
                writer.write("List.iterator(); i.hasNext(); ) {\n");
                    
                // Determine if this was a collapsed type
                if (ClassUtils.isJavaPrimitive(property.getJavaType())) {
                    collapsedElements.add(property);
                    
                    writer.write("            ");
                    writer.write(property.getJavaType());
                    writer.write(" ");
                    writer.write(property.getJavaVariableName());
                    writer.write(" = (");
                    writer.write(property.getJavaType());
                    writer.write(")i.next();\n");
                    
                    writer.write("            writer.write(");
                    writer.write("new StringBuffer(indent)");
                    writer.write(".append(\"  \").toString());\n");
                    writer.write("            writer.write(\"<");
                    writer.write(property.getXMLName());
                    writer.write(">\");\n");
                    writer.write("            writer.write(");
                    writer.write(property.getJavaVariableName());
                    writer.write(");\n");
                    writer.write("            writer.write(\"</");
                    writer.write(property.getXMLName());
                    writer.write(">\\n\");\n");
                } else {
                    writer.write("            ");
                    writer.write(property.getJavaType());
                    writer.write("Impl ");
                    writer.write(property.getJavaVariableName());
                    writer.write(" = (");
                    writer.write(property.getJavaType());
                    writer.write("Impl)i.next();\n");
                    writer.write("            ");
                    writer.write(property.getJavaVariableName());
                    writer.write(".writeXMLRepresentation(writer,\n");
                    writer.write("                new StringBuffer(");
                    writer.write("indent).append(\"  \").toString());\n");
                }
                writer.write("        }\n");
            } else {
                writer.write("        if (");
                writer.write(property.getJavaVariableName());
                writer.write(" != null) {\n");
                    
                // Determine if this was a collapsed type
                if (ClassUtils.isJavaPrimitive(property.getJavaType())) {
                    collapsedElements.add(property);
                    
                    writer.write("            writer.write(");
                    writer.write("new StringBuffer(indent)");
                    writer.write(".append(\"  \").toString());\n");
                    writer.write("            writer.write(\"<");
                    writer.write(property.getXMLName());
                    writer.write(">\");\n");
                    writer.write("            writer.write(this.");
                    writer.write(property.getJavaVariableName());
                    writer.write(");\n");
                    writer.write("            writer.write(\"</");
                    writer.write(property.getXMLName());
                    writer.write(">\\n\");\n");
                } else {
                    writer.write("            ((");
                    writer.write(property.getJavaType());
                    writer.write("Impl)");
                    writer.write(property.getJavaVariableName());
                    writer.write(").writeXMLRepresentation(writer,\n");
                    writer.write("                new StringBuffer(");
                    writer.write("indent).append(\"  \").toString());\n");
                }
                writer.write("        }\n\n");
            }
        }
        if (hasChildren) {
            writer.write("        writer.write(indent);\n");
        }
        
        // Write out the simple value, if present
        if (xmlValueVariable != null) {
            writer.write("        // Handle PCDATA value\n");
            writer.write("        writer.write(get");
            writer.write(CapitalizationUtils.initialUpper(xmlValueVariable));
            writer.write("());\n");
        }
        
        // Close up this element
        writer.write("        writer.write(\"</");
        writer.write(container.getXMLName());
        writer.write(">\\n\");\n");
        writer.write("    }\n\n");
        
        // Write out properties for needed collapsed child elements
        for (Iterator i = collapsedElements.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            writer.write("    /** Indicates if we are in the ");
            writer.write(property.getJavaVariableName());
            writer.write(" element */\n");
            writer.write("    private boolean zeus_in");
            writer.write(CapitalizationUtils.initialUpper(
                property.getJavaVariableName()));
            writer.write(" = false;\n\n");
            
            if (property.isCollection()) {
            writer.write("    /** Value for the current ");
            writer.write(property.getJavaVariableName());
            writer.write(" being processed */\n");
            writer.write("    private ");
            writer.write(property.getJavaType());
            writer.write(" zeus_current");
            writer.write(CapitalizationUtils.initialUpper(
                property.getJavaVariableName()));
            writer.write(";\n\n");
            }
        }
        
        // support: escapeAttributevalue
        writer.write("    private String escapeAttributeValue(");
        writer.write("String attributeValue) {\n");
        writer.write("        String returnValue = attributeValue;\n");
        writer.write("        for (int i = 0; i < returnValue.length(); ");
        writer.write("i++) {\n");
        writer.write("            char ch = returnValue.charAt(i);\n");
        writer.write("            if (ch == '\"') {\n");
        writer.write("                returnValue = new StringBuffer()\n");
        writer.write("                    .append(returnValue.substring(");
        writer.write("0, i))\n");
        writer.write("                    .append(\"&quot;\")\n");
        writer.write("                    .append(returnValue.");
        writer.write("substring(i+1))\n");
        writer.write("                    .toString();\n");
        writer.write("            }\n");
        writer.write("        }\n");
        writer.write("        return returnValue;\n");
        writer.write("    }\n\n");
    }
    
    /**
     * <p>
     *  This will write out the implementation of the various 
     *    <code>unmarshal</code> methods supported by generated classes.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container the container to write methods for.
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplementationUnmarshalMethods(Writer writer,
                                                       Container container)
        throws IOException {
            
        String className = container.getJavaType();
        String variableName = container.getJavaVariableName();
        
        // Set an entity resolver
        writer.write("    /**\n");
        writer.write("     * <p>\n");
        writer.write("     *  This sets a SAX <code>EntityResolver</code> for");
        writer.write(" this unmarshalling process.\n");
        writer.write("     * </p>\n");
        writer.write("     *\n");
        writer.write("     * @param resolver the entity resolver ");
        writer.write("to use.\n");
        writer.write("     */\n");
        writer.write("    public static void setEntityResolver(");
        writer.write("EntityResolver resolver) {\n");
        writer.write("        entityResolver = resolver;\n");
        writer.write("    }\n\n");
        
        // Set an error handler
        writer.write("    /**\n");
        writer.write("     * <p>\n");
        writer.write("     *  This sets a SAX <code>ErrorHandler</code> for");
        writer.write(" this unmarshalling process.\n");
        writer.write("     * </p>\n");
        writer.write("     *\n");
        writer.write("     * @param handler the entity resolver ");
        writer.write("to use.\n");
        writer.write("     */\n");
        writer.write("    public static void setErrorHandler(");
        writer.write("ErrorHandler handler) {\n");
        writer.write("        errorHandler = handler;\n");
        writer.write("    }\n\n");

        // unmarshal(File file)
        writer.write("    public static ");
        writer.write(className);
        writer.write(" unmarshal(File file)");
        writer.write(" throws IOException {\n");
        writer.write("        // Delegate to the unmarshal(Reader) method\n");
        writer.write("        return unmarshal(new FileReader(file));\n");
        writer.write("    }\n\n");
        
        // unmarshal(File file, boolean validate)
        writer.write("    public static ");
        writer.write(className);
        writer.write(" unmarshal(File file, boolean validate)");
        writer.write(" throws IOException {\n");
        writer.write("        // Delegate to the unmarshal(Reader) method\n");
        writer.write("        return unmarshal(new FileReader(file), ");
        writer.write("validate);\n");
        writer.write("    }\n\n");
            
        // unmarshal(InputStream inputStream)
        writer.write("    public static ");
        writer.write(className);
        writer.write(" unmarshal(InputStream inputStream)");
        writer.write(" throws IOException {\n");
        writer.write("        // Delegate to the unmarshal(Reader) method\n");
        writer.write("        ");
        writer.write("return unmarshal(new InputStreamReader(inputStream));\n");
        writer.write("    }\n\n");
        
        // unmarshal(InputStream inputStream, boolean validate)
        writer.write("    public static ");
        writer.write(className);
        writer.write(" unmarshal(InputStream inputStream, boolean validate)");
        writer.write(" throws IOException {\n");
        writer.write("        // Delegate to the unmarshal(Reader) method\n");
        writer.write("        ");
        writer.write("return unmarshal(new InputStreamReader(inputStream), ");
        writer.write("validate);\n");
        writer.write("    }\n\n");
        
        // unmarshal(Reader reader)
        writer.write("    public static ");
        writer.write(className);
        writer.write(" unmarshal(Reader reader)");
        writer.write(" throws IOException {\n");
        writer.write("        // Delegate with default validation value\n");
        writer.write("        ");
        writer.write("return unmarshal(reader, ");
        writer.write(ZeusDefaults.DEFAULT_VALIDATION_STATE.toString());
        writer.write(");\n");
        writer.write("    }\n\n");
        
        // unmarshal(Reader reader, boolean validate)
        writer.write("    public static ");
        writer.write(className);
        writer.write(" unmarshal(Reader reader, boolean validate)");
        writer.write(" throws IOException {\n");

        // Create new instance
        writer.write("        ");
        writer.write(className);
        writer.write("Impl ");
        writer.write(variableName);
        writer.write(" = new ");
        writer.write(className);
        writer.write("Impl();\n");
        
        // Set up class heirarchy
        writer.write("        ");
        writer.write(variableName);
        writer.write(".setCurrentUNode(");
        writer.write(variableName);
        writer.write(");\n");
        writer.write("        ");
        writer.write(variableName);
        writer.write(".setParentUNode(null);\n");
            
        // Load the XML parser
        writer.write("        // Load the XML parser\n");
        writer.write("        XMLReader parser = null;\n");
        writer.write("        String parserClass = System.getProperty(");
        writer.write("\"org.xml.sax.driver\",\n");
        writer.write("            \"");
        writer.write(ZeusDefaults.DEFAULT_PARSER);
        writer.write("\");\n");
        writer.write("        try {\n");
        writer.write("            parser = ");
        writer.write("XMLReaderFactory.createXMLReader(parserClass);\n\n");
        writer.write("            // Set entity resolver, if needed\n");
        writer.write("            if (entityResolver != null) {\n");
        writer.write("                parser.setEntityResolver(");
        writer.write("entityResolver);\n");
        writer.write("            }\n\n");
        writer.write("            // Set error handler, if needed\n");
        writer.write("            if (errorHandler != null) {\n");
        writer.write("                parser.setErrorHandler(");
        writer.write("errorHandler);\n");
        writer.write("            }\n\n");
        writer.write("            // Register content handler\n");
        writer.write("            parser.setContentHandler(");
        writer.write(variableName);
        writer.write(");\n");
        writer.write("        } catch (SAXException e) {\n");
        writer.write("            throw new IOException(");
        writer.write("\"Could not load XML parser: \" + \n");
        writer.write("                e.getMessage());\n");
        writer.write("        }\n\n");
        
        // Parse the document
        writer.write("        InputSource inputSource = new InputSource(");
        writer.write("reader);\n");
        writer.write("        try {\n");
        writer.write("            parser.setFeature(");
        writer.write("\"http://xml.org/sax/features/validation\", ");
        writer.write("new Boolean(validate).booleanValue());\n");
        writer.write("            parser.parse(inputSource);\n");
        writer.write("        } catch (SAXException e) {\n");
        writer.write("            throw new IOException(");
        writer.write("\"Error parsing XML document: \" + \n");
        writer.write("                e.getMessage());\n");
        writer.write("        }\n\n");
        
        // Return the resulting object
        writer.write("        // Return the resultant object\n");
        writer.write("        return ");
        writer.write(variableName);
        writer.write(";\n");
        writer.write("    }\n\n");
        
        // Support method: getParentUNode
        writer.write("    public Unmarshallable getParentUNode() {\n");
        writer.write("        return zeus_parentUNode;\n");
        writer.write("    }\n\n");
        
        // Support method: setParentUNode
        writer.write("    public void setParentUNode(");
        writer.write("Unmarshallable parentUNode) {\n");
        writer.write("        this.zeus_parentUNode = parentUNode;\n");
        writer.write("    }\n\n");
        
        // Support method: getCurrentUNode
        writer.write("    public Unmarshallable getCurrentUNode() {\n");
        writer.write("        return zeus_currentUNode;\n");
        writer.write("    }\n\n");
        
        // Support method: setCurrentUNode
        writer.write("    public void setCurrentUNode(");
        writer.write("Unmarshallable currentUNode) {\n");
        writer.write("        this.zeus_currentUNode = currentUNode;\n");
        writer.write("    }\n\n");
    }
    
    /**
     * <p>
     *  This will write out methods that implements various SAX handlers
     *    for use in unmarshalling.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container the container to write class for.
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplementationSAXHandlerMethods(Writer writer, 
                                                        Container container)
        throws IOException {
            
        String className = 
            CapitalizationUtils.initialUpper(container.getJavaName());
        String variableName = container.getJavaVariableName();

        // SAX startElement()
        writer.write("    public void startElement(String namespaceURI, ");
        writer.write("String localName,\n");
        writer.write("                             String qName, ");
        writer.write("Attributes atts)\n");
        writer.write("        throws SAXException {\n\n");
            
        writer.write("        // Feed this to the correct ContentHandler\n");
        writer.write("        Unmarshallable current = getCurrentUNode();\n");
        writer.write("        if (current != this) {\n");
        writer.write("            current.startElement(");
        writer.write("namespaceURI, localName, qName, atts);\n");
        writer.write("            return;\n");
        writer.write("        }\n\n");

        writer.write("        // See if we handle, or we delegate\n");
        writer.write("        if ((localName.equals(\"");
        writer.write(container.getXMLName());
        writer.write("\")) && (!zeus_thisNodeHandled)) {\n");
            
        writer.write("            // Handle ourselves\n");
        writer.write("            for (int i=0, len=atts.getLength(); i<len;");
        writer.write(" i++) {\n");
        writer.write("                String attName");
        writer.write("= atts.getLocalName(i);\n");
        writer.write("                String attValue = atts.getValue(i);\n");
        
        // Write out conditionals for this binding's attributes
        List properties = container.getProperties();
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            BitSet modifiers = property.getModifier();
            String propertyXMLName = property.getXMLName();
            
            // Only delegate to child elements
            if (modifiers.get(Property.SOURCE_ATTLIST)) {
                writer.write("                ");
                writer.write("if (attName.equals(\"");
                writer.write(propertyXMLName);
                writer.write("\")) {\n");
                writer.write("                    set");
                writer.write(CapitalizationUtils.initialUpper(
                    property.getJavaName()));
                writer.write("(attValue);\n");
                writer.write("                }\n");
            }
        }
        
        writer.write("            }\n");
        writer.write("            zeus_thisNodeHandled = true;\n");
        writer.write("            return;\n");
        writer.write("        } else {\n");
        writer.write("            // Delegate handling\n");
        
        // Delegation to properties
        String pcDataVariableName = null;      
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            BitSet modifiers = property.getModifier();
            String propertyXMLName = property.getXMLName();
            
            // Only delegate to child elements
            if (modifiers.get(Property.SOURCE_ELEMENT)) {
                String propertyClassName = 
                    CapitalizationUtils.initialUpper(property.getJavaName());
                String propertyVariableName = property.getJavaVariableName();
                
                writer.write("            if (localName.equals(\"");
                writer.write(propertyXMLName);
                writer.write("\")) {\n");
                    
                if (collapsedElements.contains(property)) {
                    writer.write("                this.zeus_in");
                    writer.write(CapitalizationUtils.initialUpper(
                        propertyVariableName));
                    writer.write(" = true;\n");
                } else {
                    writer.write("                ");
                    writer.write(property.getJavaType());
                    writer.write("Impl ");
                    writer.write(propertyVariableName);
                    writer.write(" = new ");
                    writer.write(property.getJavaType());
                    writer.write("Impl();\n");
                
                    // Set up this new object's delegation heirarchy
                    writer.write("                current = ");
                    writer.write("getCurrentUNode();\n");
                    writer.write("                ");
                    writer.write(propertyVariableName);
                    writer.write(".setParentUNode(current);\n");
                    writer.write("                ");
                    writer.write(propertyVariableName);
                    writer.write(".setCurrentUNode(");
                    writer.write(propertyVariableName);
                    writer.write(");\n");
                    writer.write("                this.setCurrentUNode(");
                    writer.write(propertyVariableName);
                    writer.write(");\n");
                    writer.write("                ");
                    writer.write(propertyVariableName);
                    writer.write(".startElement(namespaceURI, localName, ");
                    writer.write("qName, atts);\n");
                
                    // Add the (populated) object value
                    writer.write("                // Add this value in\n");
                    if (property.isCollection()) {
                        writer.write("                ");
                        writer.write(property.getJavaName());
                        writer.write("List.add(");
                        writer.write(propertyVariableName);
                        writer.write(");\n");
                    } else {
                        writer.write("                this.");
                        writer.write(propertyVariableName);
                        writer.write(" = ");
                        writer.write(propertyVariableName);
                        writer.write(";\n");
                    }
                }
                
                writer.write("                return;\n");
                writer.write("            }\n");
            } else if (propertyXMLName.equals(ZeusDefaults.PCDATA_XML_NAME)) {
                pcDataVariableName = property.getJavaVariableName();
            }
        }
        
        writer.write("        }\n");
        writer.write("    }\n\n");
        
        // SAX endElement()
        writer.write("    public void endElement(String namespaceURI, ");
        writer.write("String localName,\n");
        writer.write("                           String qName)\n");
        writer.write("        throws SAXException {\n\n");
            
        writer.write("        Unmarshallable current = getCurrentUNode();\n");
        writer.write("        if (current != this) {\n");
        writer.write("            current.endElement(");
        writer.write("namespaceURI, localName, qName);\n");
        writer.write("            return;\n");
        writer.write("        }\n\n");
        
        // Handle collapsed elements
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if (collapsedElements.contains(property)) {
                writer.write("        if (localName.equals(\"");
                writer.write(property.getXMLName());
                writer.write("\")) {\n");
                writer.write("            this.zeus_in");
                writer.write(CapitalizationUtils.initialUpper(
                    property.getJavaVariableName()));
                writer.write(" = false;\n");
                
                // Handle multiple properties
                if (property.isCollection()) {
                    writer.write("            add");
                    writer.write(CapitalizationUtils.initialUpper(
                        property.getJavaName()));
                    writer.write("(zeus_current");
                    writer.write(CapitalizationUtils.initialUpper(
                        property.getJavaVariableName()));
                    writer.write(");\n");
                    writer.write("            this.zeus_current");
                    writer.write(CapitalizationUtils.initialUpper(
                        property.getJavaVariableName()));
                    writer.write(" = null;\n");
                }
                
                writer.write("            return;\n");
                writer.write("        }\n\n");
            }
        }
        
        if (pcDataVariableName != null) {
            writer.write("        // Make sure we have textual value\n");
            writer.write("        if (this.");
            writer.write(pcDataVariableName);
            writer.write(" == null) {\n");
            writer.write("            this.");
            writer.write(pcDataVariableName);
            writer.write(" = \"\";\n");
            writer.write("        }\n");
        }
        
        writer.write("        Unmarshallable parent = ");
        writer.write("getCurrentUNode().getParentUNode();\n");
        writer.write("        if (parent != null) {\n");
        writer.write("            parent.setCurrentUNode(parent);\n");
        writer.write("        }\n");
        writer.write("    }\n\n");
        
        // SAX characters()
        writer.write("    public void characters(char[] ch, ");
        writer.write("int start, int len)\n");
        writer.write("        throws SAXException {\n\n");
        writer.write("        // Feed this to the correct ContentHandler\n");
        writer.write("        Unmarshallable current = ");
        writer.write("getCurrentUNode();\n");
        writer.write("        if (current != this) {\n");
        writer.write("            current.characters(ch, start, len);\n");
        writer.write("            return;\n");
        writer.write("        }\n\n");
        
        writer.write("        String text = new String(ch, start, len);\n");
        writer.write("        text = text.trim();\n");
        
        // Handle nested collapsed elements
        for (Iterator i = collapsedElements.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            
            String javaVariableName = property.getJavaVariableName();
            writer.write("        if (zeus_in");
            writer.write(CapitalizationUtils.initialUpper(
                javaVariableName));
            writer.write(") {\n");
                
            // Handle the case where the collapsed element is a collection
            if (property.isCollection()) {
                writer.write("            if (this.zeus_current");
                writer.write(CapitalizationUtils.initialUpper(
                    javaVariableName));
                writer.write(" == null) {\n");     
                writer.write("                this.zeus_current");
                writer.write(CapitalizationUtils.initialUpper(
                    javaVariableName));
                writer.write(" = text;\n");
                writer.write("            } else {\n");
                writer.write("                this.zeus_current");
                writer.write(CapitalizationUtils.initialUpper(
                    javaVariableName));
                writer.write(" = new StringBuffer(this.zeus_current");
                writer.write(CapitalizationUtils.initialUpper(
                    javaVariableName));
                writer.write(").append(text).toString();\n");
                writer.write("            }\n");
            } else {
                writer.write("            if (this.");
                writer.write(javaVariableName);
                writer.write(" == null) {\n");
                writer.write("                this.");
                writer.write(javaVariableName);
                writer.write(" = text;\n");
                writer.write("            } else {\n");
                writer.write("                this.");
                writer.write(javaVariableName);
                writer.write(" = new StringBuffer(this.");
                writer.write(javaVariableName);
                writer.write(").append(text).toString();\n");
                writer.write("            }\n");
            }
            writer.write("            return;\n");
            writer.write("        }\n\n");
        }
        
        // Handle character content, if applicable
        if (pcDataVariableName != null) {
            writer.write("        if (this.");
            writer.write(pcDataVariableName);
            writer.write(" == null) {\n");
            writer.write("            this.");
            writer.write(pcDataVariableName);
            writer.write(" = text;\n");
            writer.write("        } else {\n");
            writer.write("            this.");
            writer.write(pcDataVariableName);
            writer.write(" = new StringBuffer(this.");
            writer.write(pcDataVariableName);
            writer.write(").append(text).toString();\n");
            writer.write("        }\n");
        }
        writer.write("    }\n\n");
    }
    
    /**
     * <p>
     *  This will return the fully qualified class name of a specific
     *    <code>Binding</code>.
     * </p>
     *
     * @return <code>String</code> - the fully qualified Java type associated
     *         with the binding.
     */
    private String getFullyQualifiedClassName(Binding binding) {
        String packageName = binding.getJavaInterfacePackage();
        if ((packageName != null) && (packageName != "")) {
            return packageName + "." + binding.getJavaType();
        } else {
            return binding.getJavaType();
        }
    }

    /**
     * <p>
     *  This method will write to the writer an expression that evaluates to the
     *    value of the string value, but as the type of the property. 
     *    Only use this if the property has a primitive Java type.
     * </p>
     *
     * @param writer the output writer
     * @param property the property that defines the primitive type
     * @param value the string value for the primitive (i.e. "true" for boolean
     *        or "1.8" for float)
     */
    protected void generatePrimitiveAssignment(Writer writer,
                                               Property property,
                                               String value)
        throws IOException {
            
        String type = property.getJavaType();
        if (type.equals("boolean")) {
            writer.write("Boolean.valueOf(" + value + ").booleanValue()");
        } else if (type.equals("byte")) {
            writer.write("Byte.valueOf(" + value + ").byteValue()");
        } else if (type.equals("char")) {
            writer.write(value + ".charAt(0)");
        } else if (type.equals("double")) {
            writer.write("Double.valueOf(" + value + ").doubleValue()");
        } else if (type.equals("float")) {
            writer.write("Float.valueOf(" + value + ").floatValue()");
        } else if (type.equals("int")) {
            writer.write("Integer.valueOf(" + value + ").intValue()");
        } else if (type.equals("long")) {
            writer.write("Long.valueOf(" + value + ").longValue()");
        } else if (type.equals("short")) {
            writer.write("Short.valueOf(" + value + ").shortValue()");
        } else if (type.equals("String")) {
            writer.write(value);
        }
    }

    /**
     * <p>
     *  This will initialize this instance of <code>Generator</code>,
     *    handling tasks like directory creation.
     * </p>
     *
     * @throws <code>IllegalOutputDirectoryException</code> - when an illegal
     *         output directory has been supplied to this <code>Generator</code>
     */    
    private void initialize() throws IllegalOutputDirectoryException {        
        // Determine the output directory
        if (outputDir == null) {
            outputDir = new File(".");
        }
        
        // Ensure the output directory exists and is a directory
        if ((outputDir.exists()) && (!outputDir.isDirectory())) {
            throw new IllegalOutputDirectoryException(
                outputDir.getAbsolutePath());
        }

        // Create the directory if it does not exist
        if (!outputDir.exists()) {
            outputDir.mkdir();
        }

        initialized = true;
    }        

    /**
     * <p>
     *  This will reset the initialization state of this <code>Generator</code>
     *    instance to <code>false</code>.
     * </p>
     */    
    private void uninitialize() {
        initialized = false;
    }
    
    /**
     * <p>
     *  This will indicate if this <code>Generator</code> instance has been
     *    initialized.
     * </p>
     *
     * @return <code>boolean</code> - whether this instance is initialized.
     */
    private boolean isInitialized() {
        return initialized;
    }
    
    /**
     * <p>
     *  This will determine the directory to use for interface generation
     *    based on the interface package of a <code>{@link Container}</code>.
     * </p>
     *
     * @param container the container to obtain the directory for.
     * @return <code>File</code> - the directory for interface generation.
     */
    private File getInterfaceDir(Container container) {
        StringBuffer interfaceDirName = new StringBuffer();
        String interfacePackage = container.getJavaInterfacePackage();

        for (int i=0, len=interfacePackage.length(); i<len; i++) {
            char ch = interfacePackage.charAt(i);
            if (ch != '.') {
                interfaceDirName.append(ch);
            } else {
                interfaceDirName.append(System.getProperty("file.separator"));
            }
        }
        
        File interfaceDir = 
            new File(outputDir, interfaceDirName.toString());
        
        // Create the directory if it does not exist
        if (!interfaceDir.exists()) {
            interfaceDir.mkdirs();
        }
        
        return interfaceDir;
    }
    
    /**
     * <p>
     *  This will determine the directory to use for implementation generation
     *    based on the implementation package of a 
     *    <code>{@link Container}</code>.
     * </p>
     *
     * @param container the container to obtain the directory for.
     * @return <code>File</code> - the directory for implementation generation.
     */
    private File getImplementationDir(Container container) {
        StringBuffer implementationDirName = new StringBuffer();
        String implementationPackage = container.getJavaImplementationPackage();

        for (int i=0, len=implementationPackage.length(); i<len; i++) {
            char ch = implementationPackage.charAt(i);
            if (ch != '.') {
                implementationDirName.append(ch);
            } else {
                implementationDirName.append(
                    System.getProperty("file.separator"));
            }
        }
        
        File implementationDir = 
            new File(outputDir, implementationDirName.toString());
        
        // Create the directory if it does not exist
        if (!implementationDir.exists()) {
            implementationDir.mkdirs();
        }
        
        return implementationDir;
    }
}
