/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.transform;

import java.util.HashMap;
import java.io.IOException;
import java.util.List;
import java.util.Map;

// Zeus imports
import org.enhydra.zeus.Source;
import org.enhydra.zeus.Transformer;
import org.enhydra.zeus.ZeusException;
import org.enhydra.zeus.transform.ValueEnumeration;

/**
 * <p>
 *  <code>{@link Transformer}</code> is the interface for converting a set of
 *    Zeus <code>{@link org.enhydra.zeus.Binding}</code> objects from a set of 
 *    XML-based names and identities (created by a Zeus 
 *    <code>{@link org.enhydra.zeus.Binder}</code> instance) to another set of 
 *    <code>Binding</code> objects. This second
 *    set of objects is only an intermediate set, which is then used by a
 *    <code>{@link org.enhydra.zeus.Generator}</code> to create Java classes.
 * </p><p>
 *  This allows a user to specify a <i>mapping file</i> and thereby transform
 *    a set of XML names into another set of names, either for ensuring legal
 *    Java names, for using more business-centric names, or any other user
 *    desired basis.
 * </p><p>
 *  <code>BaseTransformer</code> is an abstract, base implementation of 
 *    <code>Transformer</code>, and provides common functionality. All
 *    <code>Transformer</code> implementations should extend this base class.
 * </p>
 *
 * @author Brett McLaughlin
 */
public abstract class BaseTransformer implements Transformer {
    
    /** The <code>{@link Source}</code> to read mapping information from */
    protected Source source;
    
    /** The <code>{@link TransformerOptions}</code> for this transformer */
    private TransformerOptions transformerOptions;
    
    /** The list of <code>ValueEnumeration</code>s for this transformer. */
    private Map valueEnumerations;
    
    /**
     * <p>
     *  This constructor takes in a <code>{@link Source}</code>
     *    to read mapping information from and allow generation of the 
     *    transformation mappings from it.
     * </p>
     *
     * @param source <code>Source</code> to read mapping input from.
     */
    public BaseTransformer(Source source) {
        // The only transformer allowed to have a null source is the default
        if ((source == null) && !(this instanceof DefaultsTransformer)) {
            throw new IllegalArgumentException("A Transformer cannot have " +
                "a null Source.");
        }
        this.source = source;
        
        // Initialize variables
        transformerOptions = new TransformerOptions();
        valueEnumerations = new HashMap();
    }
    
    /**
     * <p>
     *  This allows setting of the <code>{@link TransformerOptions}</code> for
     *    this transformer.
     * </p>
     *
     * @param transformerOptions the options for this transformer
     */
    public void setTransformerOptions(TransformerOptions transformerOptions) {
        if (transformerOptions == null) {
            throw new IllegalArgumentException("A Transformer cannot " +
                "have a null set of TransformerOptions.");
        }
        this.transformerOptions = transformerOptions;
    }
    
    /**
     * <p>
     *  This returns the current <code>{@link TransformerOptions}</code> for
     *    this transformer.
     * </p>
     *
     * @return <code>TransformerOptions</code> - the options in use for this
     *         transformer.
     */
    public TransformerOptions getTransformerOptions() {
        return transformerOptions;
    }
    
    /**
     * <p>
     *  This will add a new <code>{@link ValueEnumeration}</code> to those
     *    available to this <code>Transformer</code>.
     * </p>
     *
     * @param valueEnumeration the enumeration to add for this transformer.
     */
    public void addValueEnumeration(ValueEnumeration valueEnumeration) {
        if (valueEnumeration == null) {
            throw new IllegalArgumentException("A Transformer cannot have " +
                "a null ValueEnumeration.");
        }
        valueEnumerations.put(valueEnumeration.getName(), valueEnumeration);
    }
    
    /**
     * <p>
     *  This will retrieve the <code>{@link ValueEnumeration}</code> for the
     *    supplied name.
     * </p><p>
     *  If the supplied name is not associated with a 
     *    <code>ValueEnumeration<code>, this is considered an <i>exceptional</i>
     *    condition; any <code>Transformer</code> construct looking for a
     *    <code>ValueEnumeration</code> by name <i>expects</i> to find one,
     *    or the supplied mapping information is in error. Therefore, 
     *    in this case, a <code>{@link NoSuchValueEnumerationException}</code>
     *    is thrown.
     * </p>
     *
     * @return <code>ValueEnumeration</code> - the enumeration for the supplied
     *         name.
     * @throws <code>NoSuchValueEnumerationException</code> - when an
     *         enumeration with the supplied name cannot be found.
     */
    public ValueEnumeration getValueEnumeration(String name) 
        throws NoSuchValueEnumerationException {
            
        ValueEnumeration valueEnumeration = 
            (ValueEnumeration)valueEnumerations.get(name);
            
        // If no enumeration found, throw an exception
        if (valueEnumeration == null) {
            throw new NoSuchValueEnumerationException(name);
        }
        
        // If we got here, return the located enumeration
        return valueEnumeration;
    }
    
    /**
     * <p>
     *  This method performs the work of transforming a set of Zeus
     *    <code>{@link org.enhydra.zeus.Binding}</code> objects. It then 
     *    returns a <i>new</i> set of <code>Binding</code> objects 
     *    (in <code>List</code> form) with transformed names, types, and values.
     * </p><p>
     *  When this method is invoked, it attempts to see if parsing of the 
     *    mapping file has already occurred. If it has not, parsing of that
     *    file will occur within this method.
     * </p><p>
     *  It is important to note that the <i>number</i> of bindings returned
     *    from this method may not be the same as the number supplied. This
     *    is due to some XML elements/attributes being converted to value
     *    objects, while other mapping instructions may actually create new
     *    binding objects.
     * </p>
     *
     * @param bindings <code>List</code> of bindings to process for
     *        transformation.
     * @return <code>List</code> - the transformed bindings.
     * @throws <code>IOException</code> - when errors occur in parsing
     *         a mapping configuration file.
     * @throws <code>ZeusException</code> - when errors occuring in
     *         transformation.
     */
    public List transform(List bindings) throws IOException, ZeusException {
        // Perform transformation, without recursion
        return transform(bindings, false);
    }
    
    /**
     * <p>
     *  This method performs the work of transforming a set of Zeus
     *    <code>{@link org.enhydra.zeus.Binding}</code> objects. It then 
     *    returns a <i>new</i> set of <code>Binding</code> objects 
     *    (in <code>List</code> form) with transformed names, types, and values.
     * </p><p>
     *  When this method is invoked, it attempts to see if parsing of the 
     *    mapping file has already occurred. If it has not, parsing of that
     *    file will occur within this method.
     * </p><p>
     *  It is important to note that the <i>number</i> of bindings returned
     *    from this method may not be the same as the number supplied. This
     *    is due to some XML elements/attributes being converted to value
     *    objects, while other mapping instructions may actually create new
     *    binding objects.
     * </p>
     *
     * @param bindings <code>List</code> of bindings to process for
     *        transformation.
     * @param recursing indication of if this is within a recursion loop.
     * @return <code>List</code> - the transformed bindings.
     * @throws <code>IOException</code> - when errors occur in parsing
     *         a mapping configuration file.
     * @throws <code>ZeusException</code> - when errors occuring in
     *         transformation.
     */
    public abstract List transform(List bindings, boolean recursing) 
        throws IOException, ZeusException;

}
