/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.transform;

import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

// Zeus imports
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.binding.Container;
import org.enhydra.zeus.Source;
import org.enhydra.zeus.ZeusDefaults;
import org.enhydra.zeus.ZeusException;
import org.enhydra.zeus.util.ClassUtils;
import org.enhydra.zeus.util.NamingUtils;

/**
 * <p>
 *  <code>SimpleTransformer</code> is a basic implementation of a Zeus
 *    <code>{@link org.enhydra.zeus.Transformer}</code>. It iterates through
 *    the supplied <code>{@link Binding}</code> objects and assigns each the
 *    options for this transformer.
 * </p>
 *
 * @author Brett McLaughlin
 */
public class SimpleTransformer extends BaseTransformer {
    
    /**
     * <p>
     *  This constructor takes in a <code>{@link Source}</code>
     *    to read mapping information from and allow generation of the 
     *    transformation mappings from it.
     * </p>
     *
     * @param source <code>Source</code> to read mapping input from.
     */
    public SimpleTransformer(Source source) {
        super(source);
    }
    
    /**
     * <p>
     *  This method performs the work of transforming a set of Zeus
     *    <code>{@link org.enhydra.zeus.Binding}</code> objects. It then 
     *    returns a <i>new</i> set of <code>Binding</code> objects 
     *    (in <code>List</code> form) with transformed names, types, and values.
     * </p><p>
     *  When this method is invoked, it attempts to see if parsing of the 
     *    mapping file has already occurred. If it has not, parsing of that
     *    file will occur within this method.
     * </p><p>
     *  It is important to note that the <i>number</i> of bindings returned
     *    from this method may not be the same as the number supplied. This
     *    is due to some XML elements/attributes being converted to value
     *    objects, while other mapping instructions may actually create new
     *    binding objects.
     * </p>
     *
     * @param bindings <code>List</code> of bindings to process for
     *        transformation.
     * @param recursing indication of if this is within a recursion loop.
     * @return <code>List</code> - the transformed bindings.
     * @throws <code>IOException</code> - when errors occur in parsing
     *         a mapping configuration file.
     * @throws <code>ZeusException</code> - when errors occuring in
     *         transformation.
     */
    public List transform(List bindings, boolean recursing) 
        throws IOException, ZeusException {
            
        // Create list for result bindings
        List resultBindings = new LinkedList();
        List usedNames = new LinkedList();
        List usedVariableNames = new LinkedList();
        
        // Assign Java properties from TransformerOptions to each binding
        for (Iterator i = bindings.iterator(); i.hasNext(); ) {
            Binding binding = (Binding)i.next();
            
            if (!recursing) {
                usedNames.clear();
                usedVariableNames.clear();
            }
            
            // Handle generic options
            
            // This is temporary //
            int count = 2;
            String javaName = NamingUtils.getJavaName(binding.getXMLName());
            while (usedNames.contains(javaName)) {
                javaName = new StringBuffer(javaName)
                    .append(count)
                    .toString();
                count++;
            }
            usedNames.add(javaName);
            binding.setJavaName(javaName);
            
            count = 2;
            String javaVariableName = 
                NamingUtils.getJavaVariableName(binding.getXMLName());
            while (usedVariableNames.contains(javaVariableName)) {
                javaVariableName = new StringBuffer(javaVariableName)
                    .append(count)
                    .toString();
                count++;
            }
            usedVariableNames.add(javaVariableName);
            binding.setJavaVariableName(javaVariableName);
                
            binding.setJavaType(
                NamingUtils.getJavaType(binding.getXMLType()));
            // End temporary block //

            // Only set the package for non-PCDATA types
            if (!ClassUtils.isJavaPrimitive(binding.getJavaType())) {
                binding.setJavaInterfacePackage(
                    getTransformerOptions().getInterfacePackage());
                binding.setJavaImplementationPackage(
                    getTransformerOptions().getImplementationPackage());
            } else {
                binding.setJavaInterfacePackage("");
                binding.setJavaImplementationPackage("");
            }
            
            binding.setJavaCollectionClass(
                getTransformerOptions().getDefaultCollectionType());
            
            // If this is a container, handle children
            if (binding instanceof Container) {
                Container container = (Container)binding;
                // Recurse
                container.setProperties(
                    transform(container.getProperties(), true));
            }
            
            // Add the transformed bindings to the result set
            resultBindings.add(binding);
        }
        
        return resultBindings;
    }

}
