/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.util;

import java.util.HashSet;

// Zeus imports
import org.enhydra.zeus.InvalidCollectionTypeException;

/**
 * <p>
 *  <code>ClassUtils</code> is a Zeus utility class that provides
 *    methods that deal with the various classes and types that
 *    may be used in data binding. Specifically, this will deal
 *    with the <code>String</code> Java types that 
 *    <code>{@link org.enhydra.zeus.Generator}</code>s deal with,
 *    and relate them to Java <code>Class</code> objects.
 * </p>
 *
 * @author Brett McLaughlin
 */
public class ClassUtils {
    
    /** A hashed set of the Java primitive types. */
    private static final HashSet primitives;
    
    /** Constant for using List as the default collection type */
    public static final int COLLECTION_TYPE_LIST  = 0;
    
    /** Constant for using Array as the default collection type */
    public static final int COLLECTION_TYPE_ARRAY = 1;
    
    static {
        /** Initialize the primitives hashed set. */
        primitives = new HashSet();
        primitives.add("boolean");
        primitives.add("byte");
        primitives.add("char");
        primitives.add("double");
        primitives.add("float");
        primitives.add("int");
        primitives.add("long");
        primitives.add("short");
        primitives.add("String");
        primitives.add("java.lang.String");
    }
    
    /**
     * <p>
     *  This will indicate whether the supplied Java type is a Java
     *    primitive.
     * </p>
     *
     * @param javaType the Java type to check against Java primitives
     * @return <code>boolean</code> whether the supplied type is a Java 
     *         primitive
     */
    public static boolean isJavaPrimitive(String javaType) {
        if (javaType == null) {
            throw new IllegalArgumentException("A non-null String must be " +
                "supplied to ClassUtils methods.");
        }
        
        return primitives.contains(javaType);
    }

    /**
     * <p>
     *  This will determine if the supplied Java type (in
     *    <code>String</code> form) is one of the Java
     *    <code>Collection</code> classes. The type supplied <i>must</i>
     *    be a fully-qualified class name, like <code>java.util.List</code>.
     * </p>
     *
     * @param javaType the <code>String</code> name of the Java
     *        type to check for.
     * @return <code>boolean</code> - whether the class is a
     *         Java <code>Collection</code>.
     */
    public static boolean isCollectionClass(String javaType) {
        if (javaType == null) {
            throw new IllegalArgumentException("A non-null String must be " +
                "supplied to ClassUtils methods.");
        }
        
        // Since all classes are fully qualified, check for java.util package
        if (javaType.indexOf("java.util") == -1) {
            return false;
        }
        
        // Check for the specific Collection names, in order of commonality
        if (javaType.indexOf("Collection") != -1) {
            return true;
        } else if (javaType.indexOf("List") != -1) {
            return true;            
        } else if (javaType.indexOf("LinkedList") != -1) {
            return true;
        } else if (javaType.indexOf("ArrayList") != -1) {
            return true;
        } else if (javaType.indexOf("Vector") != -1) {
            return true;
        } else if (javaType.indexOf("Stack") != -1) {
            return true;
        } else if (javaType.indexOf("Set") != -1) {
            return true;  
        } else if (javaType.indexOf("SortedSet") != -1) {
            return true;
        } else if (javaType.indexOf("HashSet") != -1) {
            return true;                 
        } else if (javaType.indexOf("TreeSet") != -1) {
            return true;                 
        }
        
        return false;                                                
    }
    
    /** 
     * <p>
     *  For a supplied <code>Collection</code> class, this will return an 
     *    implementation of that class. For example, specifying 
     *    <code>java.util.List</code> to this method would return 
     *    <code>java.util.LinkedList</code>, which is a concrete class. If the
     *    supplied class is already a concrete class, it is returned unchanged.
     * </p>
     *
     * @param collectionClass <code>String</code> name of collection 
     *        class/interface
     * @return <code>String</code> - fully qualified concrete class to use.
     */
    public static String getCollectionImplClass(String collectionClass) {
        if (collectionClass == null) {
            throw new IllegalArgumentException("A non-null String must be " +
                "supplied to ClassUtils methods.");
        }
        
        // Check for the specific Collection names, in order of commonality
        if (collectionClass.indexOf("Collection") != -1) {
            return "java.util.LinkedList";
        } else if (collectionClass.indexOf("LinkedList") != -1) {
            return collectionClass;
        } else if (collectionClass.indexOf("ArrayList") != -1) {
            return collectionClass;
        } else if (collectionClass.indexOf("Vector") != -1) {
            return collectionClass;
        } else if (collectionClass.indexOf("Stack") != -1) {
            return collectionClass;
        } else if (collectionClass.indexOf("SortedSet") != -1) {
            return collectionClass;
        } else if (collectionClass.indexOf("HashSet") != -1) {
            return collectionClass;                 
        } else if (collectionClass.indexOf("TreeSet") != -1) {
            return collectionClass;                 
        } else if (collectionClass.indexOf("Set") != -1) {
            return "java.util.HashSet";  
        } else if (collectionClass.indexOf("List") != -1) {
            return "java.util.LinkedList";            
        } else {
            throw new IllegalArgumentException(
                "A non Collection class was specified.");
        }
    }
    
    /**
     * <p>
     *  This checks to see if the supplied <code>int</code> constant is a
     *    legal collection type, as defined in this class.
     * </p>
     *
     * @param collectionType <code>int</code> to check against constants.
     * @return <code>boolean</code> - whether the supplied type is a legal
     *         collection type constant.
     */
    public static boolean isCollectionConstant(int collectionType) {
        // Make sure the constant is within the correct boundary points
        if ((collectionType < COLLECTION_TYPE_LIST) ||
            (collectionType > COLLECTION_TYPE_ARRAY)) {
                
            return false;
        }
            
        return true;
    }
    
    /**
     * <p>
     *  This will convert a <code>String</code> representation of a supplied
     *    collection type, and convert it to one of the <code>int</code>
     *    constants specified in this class.
     * </p>
     *
     * @param collectionTypeString the collection type to convert to a constant.
     * @return <code>int</code> - the constant representation of the type.
     * @throws <code>InvalidCollectionTypeException</code> - when an illegal
     *         collection type is supplied.
     * @see <code>{@link #COLLECTION_TYPE_LIST}</code>
     * @see <code>{@link #COLLECTION_TYPE_ARRAY}</code>
     */
    public static int getCollectionTypeAsInt(String collectionTypeString) 
        throws InvalidCollectionTypeException {
        
        if ((collectionTypeString.equals("List")) ||
            (collectionTypeString.equals("java.util.List"))) {
            
            return COLLECTION_TYPE_LIST;
        } else if ((collectionTypeString.equals("Array")) ||
                   (collectionTypeString.equals("java.util.Array"))) {
                       
            return COLLECTION_TYPE_ARRAY;
        } else {
            throw new InvalidCollectionTypeException(collectionTypeString);
        }
    }
    
    /**
     * <p>
     *  This returns the currently selected default collection type, as a
     *    <code>String</code> that maps to a Java class. For example, if
     *    the default collection type is currently "List", this would return
     *    the value "java.util.List". The class name returned here is always
     *    a fully qualified class name.
     * </p>
     *
     * @return <code>String</code> - the default collection type as a Java class
     *         name.
     */
    public static String getCollectionTypeAsString(int collectionType) {
        if (collectionType == COLLECTION_TYPE_LIST) {
            return "java.util.List";
        } else if (collectionType == COLLECTION_TYPE_ARRAY) {
            return "java.util.Array";
        }
        
        // We should never get here
        return null;
    }
    
    /**
     * <p>
     *  This will take the <code>String</code> value supplied and convert it
     *    to an <code>Object</code> of the type specified in 
     *    <code>paramType</code>.
     * </p>
     *
     * @param value <code>String</code> value to convert.
     * @param paramType <code>Class</code> with type to convert to.
     * @return <code>Object</code> - value in correct type.
     */
    public static Object getParameter(String value, Class paramType) {
        if (value == null) {
            throw new IllegalArgumentException("A non-null String must be " +
                "supplied to ClassUtils methods.");
        }
        if (paramType == null) {
            throw new IllegalArgumentException("A non-null Class must be " +
                "supplied to ClassUtils methods.");
        }
        
        Object ob = null;
        String type = paramType.getName();

        if (type.equals("java.lang.String")) {
            ob = value;
        } else if ((type.equals("int")) || 
                   (type.equals("java.lang.Integer"))) {
            ob = Integer.valueOf(value);
        } else if ((type.equals("long")) || 
                   (type.equals("java.lang.Long"))) {
            ob = Long.valueOf(value);
        } else if ((type.equals("float")) || 
                   (type.equals("java.lang.Float"))) {
            ob = Float.valueOf(value);
        } else if ((type.equals("double")) || 
                   (type.equals("java.lang.Double"))) {
            ob = Double.valueOf(value);
        } else if ((type.equals("boolean")) || 
                   (type.equals("java.lang.Boolean"))) {
            ob = Boolean.valueOf(value);
        }

        return ob;
    }    
}
