/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.source;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;

// Zeus imports
import org.enhydra.zeus.Source;

// DTD Parser imports
import com.wutka.dtd.DTD;

// JUnit imports
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * <p>
 *  <code>StreamDTDSourceTest</code> provides rudimentary tests for the
 *    <code>{@link StreamDTDSource}</code> class. 
 * </p>
 *
 * @author Brett McLaughlin
 */
public class StreamDTDSourceTest extends TestCase {

    /**
     * <p>
     *  Creates a new <code>StreamDTDSourceTest</code> instance.
     *    Required by JUnit. <CODE>testName</CODE>
     *    is simply the name of the test.
     * </p>
     *
     * @param testName a <code>String</code> value
     */
    public StreamDTDSourceTest(String testName) {
        super(testName);
    }

    /**
     * <p>
     *  This method maeks it easier to call these
     *    tests dynamically.
     * </p>
     *
     * @return <code>TestSuite</code> - corresponds to this 
     *         <code>TestCase</code>.
     */
    public static Test suite() {
        return new TestSuite(StreamDTDSourceTest.class);
    }

    /**
     * <p>
     *  Test the various <CODE>{@link StreamDTDSource}</CODE> 
     *    constructors.
     * </p>
     */
    public void testConstructors() {
        String systemID = "file:///test/system/ID.xml";
        FileInputStream in = null;
        try {
            in =  new FileInputStream(new File("build.sh"));
        } catch (FileNotFoundException neverHappens) { }
        StreamDTDSource ss = new StreamDTDSource(in, systemID);
        ss = new StreamDTDSource(in);
        
        StringReader reader = new StringReader("Test Input Reader");
        ss = new StreamDTDSource(reader, systemID);
        
        ss = new StreamDTDSource(reader);
    }
    
    /**
     * <p>
     *  Rudimentary test of the 
     *    <CODE>{@link StreamDTDSource#getDTD()}</CODE> method.
     *    This is a negative test. Getting the DTD should
     *    fail and this should be detected and handled correctly. 
     * </p>
     */
    public void testGetDTDNegative() {
        boolean caught = false;
        try {
            String systemID = "file:///test/system/ID.dtd";
            FileInputStream in = null;
            try {
                in =  new FileInputStream(new File("build.sh"));
            } catch (FileNotFoundException neverHappens) { }
            StreamDTDSource ss = new StreamDTDSource(in, systemID);
            DTD dtd = ss.getDTD();
        } catch (IOException e) {
            caught = true;
        } finally {
            assertTrue(caught);
        }
    }

    /**
     * <p>
     *  Rudimentary positive test of the 
     *    <CODE>{@link StreamDTDSource#getDTD()}</CODE> method.
     *    A viable DTD should be returned.
     * </p>
     */
    public void testGetDTDPositive() {
        // I need a robust way to get an XML DTD here
        // What is the plan for system properties type configuration?
    }


    /**
     * <p>
     *  Several small tests that exercise 
     *    <CODE>{@link StreamDTDSource#getSystemID()}</CODE>.
     * </p>
     */
    public void testGetSystemID() {
        String systemID = "file:///test/system/ID.xml";
        FileInputStream in = null;
        try {
            in =  new FileInputStream(new File("build.sh"));
        } catch (FileNotFoundException neverHappens) { }
        StreamDTDSource ss = new StreamDTDSource(in, systemID);
        assertEquals(systemID, ss.getSystemID());
        
        ss = new StreamDTDSource(in);
        assertNull(ss.getSystemID());

        StringReader reader = new StringReader("Test Input Reader");
        ss = new StreamDTDSource(reader, systemID);
        assertEquals(systemID, ss.getSystemID());

        ss = new StreamDTDSource(reader);
        assertNull(ss.getSystemID());

    }
    
    /**
     * <p>
     *  Several small tests that exercise 
     *    <CODE>{@link StreamDTDSource#setSystemID()}</CODE>.
     * </p>
     */
    public void testSetSystemID() {
        String systemID = "file:///test/system/ID.xml";
        FileInputStream in = null;
        try {
            in =  new FileInputStream(new File("build.sh"));
        } catch (FileNotFoundException neverHappens) { }
        StreamDTDSource ss = new StreamDTDSource(in);
        assertNull(ss.getSystemID());
        
        ss.setSystemID(systemID);
        assertEquals(systemID, ss.getSystemID());

        ss = new StreamDTDSource(in, systemID);
        assertEquals(systemID, ss.getSystemID());

        StringReader reader = new StringReader("Test Input Reader");
        ss = new StreamDTDSource(reader);
        assertNull(ss.getSystemID());
        ss.setSystemID(systemID);
        assertEquals(systemID, ss.getSystemID());
    }
}
